<?php

function ip_list_vcl_callback() {
  header('Content-Type: text/plain');
  echo ip_list_db_to_acls();
  exit();
}

/**
 * Convert unexpired IP ranges from the DB to ACLs.
 */
function ip_list_db_to_acls() {
  $acls = array();
  $res = db_query('SELECT range_start AS begin, range_end AS end, organization FROM {ip_list} WHERE expires IS NULL OR expires = 0 OR expires > %d ORDER BY range_start', time());
  while ($row = db_fetch_array($res)) {
    // TODO: Detect and merge overlapping and adjacent ranges.
    $acls = array_merge($acls, ip_list_range_to_acls($row['begin'], $row['end'], $row['organization']));
  }

  return 'acl ip_list {' . PHP_EOL . implode(PHP_EOL, $acls) . PHP_EOL . '}' . PHP_EOL;
}

/**
 * Convert an arbitrary IP range into optimal Varnish ACL entries.
 *
 * Beginning and ending IPs should be in ip2long format.
 */
function ip_list_range_to_acls($begin, $end, $organization = NULL) {
  $acls = array();

  if (isset($organization)) {
    $acls[] = '  // ' . $organization;
  }

  // Loop as long as we still have IPs left to add.
  while ($begin <= $end) {
    // Find the optimal CIDR notation to eat as much
    // of the remaining IP set as possible.
    for ($cidr = 8; $cidr <= 32; ++$cidr) {
      // Apply the mask to determine the minimum and maximum IPs it would specify.
      $mask = _ip_list_cidr_to_mask($cidr);
      $max = $begin | $mask;
      $min = $max - $mask;

      // The first CIDR mask that doesn't extend outside our intended
      // range is the ideal one to pick.
      if ($min >= $begin && $max <= $end) {
        // Add the ACL.
        if ($cidr < 32) {
          $acls[] = '  "' . long2ip($begin) . '"/' . $cidr . ';' ;
        }
        else {
          $acls[] = '  "' . long2ip($begin) . '";';
        }

        // Start the new beginning IP to just after the block we
        // added with the notation above.
        $begin = $max + 1;

        // Set to 33 to avoid further CIDR checks with the current
        // beginning IP address.
        $cidr = 33;
      }
    }
  }

  return $acls;
}

function _ip_list_cidr_to_mask($cidr) {
  $mask = 0;
  for ($i = 0; $i < (32 - $cidr); ++$i) {
    $mask += (1 << $i);
  }
  return $mask;
}
